﻿/*
    
   Author: Mikko Vestola
   http://www.mvnet.fi
   mvnet@mvnet.fi

   ==========================================
   
    This file is part of Viivi & Wagner widget.

    Viivi & Wagner widget is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Viivi & Wagner widget is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Viivi & Wagner widget.  If not, see <http://www.gnu.org/licenses/>.	

   ==========================================

    Viivi ja Wagner -widgetti:    
    Widgetti näyttää päivän Viivi ja Wagner-sarjakuvan
    Helsingin Sanomien verkkosivulta.
	

*/

// Tulostetaanko debug-ikkunaan tietoja sarjakuvan latauksesta
var debug = false;

/* Näytetäänkö koko sarjakuvaa vai vain ensimmäinen strippi, Helsingin Sanomien vaatimuksesta.
 True tarkoittaa että näytetään vain ensimmäinen strippi. */
var censor = true;

// Sarjakuvan URL-osoitteen URL-olio
var urlObj = new URL();

// Helsingin sanomien verkkosivujen lukemista varten tarvittava XMLHTTP-olio
var xmlhttp;

// Tästä osoitteesta haetaan tietoa. Sivulta löytyy päivän Viivi ja Wagner -kuvan osoite, jota tarvitaan.
var url = "http://www.hs.fi/viivijawagner/";

// Regular expression, jota etsitään HTML-sivulta (sisältää sarjakuvan kuvan URL-osoitteen)
var comicRegExp = /<div[\s\S]*?class="strip"[\s\S]*?>[\s\S]*?img[\s\S]*?src="(.*?)"/; 


/*
 *  Lokalisoi ohjelman. Käännökset eri kielille löytyvät Resources-kansiosta.
*/
function localize() {

	preferences.updateInterval.title = widget.getLocalizedString("updateInterval_title");
	preferences.updateInterval.description = widget.getLocalizedString("updateInterval_description");
	
	preferences.headsUp.title = widget.getLocalizedString("headsUp_title");
	preferences.headsUp.description = widget.getLocalizedString("headsUp_description");

	preferences.manually.title = widget.getLocalizedString("manually_title");
	preferences.manually.description = widget.getLocalizedString("manually_description");
	
	lastUpdated.data = widget.getLocalizedString("lastUpdated")+"N/A";
	crJuba.data = widget.getLocalizedString("crJuba");
	crHS.data = widget.getLocalizedString("crHS");
		
}

/*
 * Päivittää asetukset.
 */
 
function updatePreferences()
{
	// Jos on valittu, että sarjakuva päivitetään vain heijastusnäytön avautuessa tai manuaalisesti, pysäytetään ajastin
	if (preferences.headsUp.value == 1 || preferences.manually.value == 1)
		timer1.ticking = false;
	else
		timer1.ticking = true;
		
	// Asetetaan ajastimen päivitysväliksi se, mikä asetuksiin on annettu (muutetaan sekunneista tunneiksi)
	timer1.interval = preferences.updateInterval.value*60*60;
	
}


/* Päivittää sarjakuvan (feidaa kuvan ulos ja näyttää spinner-kuvan) */
function updateImage() {

	// Päivitä-ikonin kuvan muuttaminen 
	if (refresh.src != "Images/refresh.png")
		refresh.src = "Images/refresh_mouseclick.png";

	// Feidaa kuva ulos ja näytä spinner-kuva
	if (comicStrip.src != "Images/loading.png") {
		fadeOut(comicStrip);
	}
	
	if (censor) {
		fadeOut(seeWhole);
		fadeOut(hsfilogo);
	}
	
	spinner.visible = true;

	// Lataa uusi sarjakuva
	loadImage();

}


/* Tekee objektista läpinäkyvämmän */
function fadeOut(textObject) {
	animator.start(new FadeAnimation(textObject, 100, 600, animator.kEaseOut));
}

/* Poistaa objektin läpinäkyvyyden */
function fadeIn(textObject) {
	animator.start(new FadeAnimation(textObject, 255, 1000, animator.kEaseOut));
}

/* Asettaa päivitä-ikonin mouseover-tilaan */
function refreshIconOver() {
	refresh.src = "Images/refresh_mouseover.png";
}

/* Asettaa päivitä-ikonin normaaliin tilaan */
function refreshIconExit() {
	refresh.src = "Images/refresh.png";
}

/* Asettaa päivitä-ikonin mouseover-tilaan (tai jättää sen normaaliin tilaan) kun käyttäjä on päästänyt hiiren painikkeen */
function refreshIconUp() {
	if (refresh.src != "Images/refresh.png")
		refresh.src = "Images/refresh_mouseover.png";
}

/* Suoritetaan, kun heijastusnäyttö avataan */
function konsposeActivated() {
	// Sarjakuva ladataan aina uudestaan, jos vain manuaalista päivitystä ei ole asetettu
	if (preferences.manually.value != 1)
		updateImage();
}

/* Tämä on Helsingin Sanomien vaatimuksesta, koko sarjakuvaa ei saa näyttää. */
function performCensor(object) {

	object.clipRect = "0, 0, 200, 207";
	seeWhole.data = widget.getLocalizedString("seeWhole");
	permissionInfo.data = widget.getLocalizedString("permissionInfo");
	hsfilogo.visible = true;	
	fadeIn(seeWhole);
	fadeIn(hsfilogo);
}


/*
 * Funktio, joka suoritetaan, kun asynkroninen fetch on suoritettu parametrin URL-oliolle (kuvan hakeminen palvelimelta)
 */ 
function fetchDone(urlObj) {

	comicStrip.src = urlObj.outputFile;
	comicStrip.reload();
	
	// Asetetaan sarjakuvan osoite äsken luetuksi ja sijoitetaan se hieman eri paikkaan
	comicStrip.hOffset = 15;
	comicStrip.vOffset = 37;
	
	// Helsingin Sanomien vaatimuksesta
	if (censor)
		performCensor(comicStrip);
		
	fadeIn(comicStrip);
	spinner.visible = false;

	if (debug)
		print("Fetch: "+urlObj.location);
		
	// Lasketaan tämänhetkinen päiväys ja kellonaika
	var currentTime = new Date();
	var month = currentTime.getMonth() + 1
	var day = currentTime.getDate();
	var year = currentTime.getFullYear();
	var hour = currentTime.getHours();
	var min = currentTime.getMinutes();
	var sec = currentTime.getSeconds();
	
	var dayNumber = currentTime.getDay();
	var dayName = "";
	
	if (dayNumber==1)
		dayName = widget.getLocalizedString("day_Mon");
	else if (dayNumber==2)
		dayName = widget.getLocalizedString("day_Tue");
	else if (dayNumber==3)
		dayName = widget.getLocalizedString("day_Wed");
	else if (dayNumber==4)
		dayName = widget.getLocalizedString("day_Thu");
	else if (dayNumber==5)
		dayName = widget.getLocalizedString("day_Fri");
	else if (dayNumber==6)
		dayName = widget.getLocalizedString("day_Sat");
	else if (dayNumber==0)
		dayName = widget.getLocalizedString("day_Sun");
	
	// Tunnit, minuutit ja sekunnit halutaan näyttää aine kahdella numerolla.
	if (hour < 10)
		hour = "0"+hour;
	if (min < 10)
		min = "0"+min;
	if (sec < 10)
		sec = "0"+sec;
	
	// Päivitetään "Viimeksi päivitetty"-kentän arvo vastaamaan aikaleimaa
	lastUpdated.data = widget.getLocalizedString("lastUpdated")+dayName+" "+
	day+"."+month+"."+year+" "+hour+":"+min+":"+sec;

}

/*
 * Lataa viivi ja vagner -sarjakuvan Helsingin Sanomien verkkosivuilta ja laittaa sen näkyville widgettiin.
 */
 
function loadImage() {
   
   // Luodaan uusi http-yhteys, jolla haetaan tietoa palvelimelta
   xmlhttp = new XMLHttpRequest();
   xmlhttp.setRequestHeader("Cache-Control", "no-cache");
   
   // Määritellään funktio, joka suoritetaan, kun http-yhteyden tila muuttuu (itse funktion rakenne on määritelty alla)
   xmlhttp.onreadystatechange=state_Change;
     
  // Avataan yhteys palvelimelle edellä määriteltyyn osoitteeseen ja lähetetään pyyntö palvelimelle.
  xmlhttp.open("GET", url, true);
  xmlhttp.send(null);
  
  // Tätä funktiota kutsutaan, kun http-yhteyden tila muuttuu
  function state_Change() {
	
		// Http-yhteyden tila 4 vastaa sitä, että palvelimelta on saatu vastaus. Muista tiloista ei välitetä.
		if (xmlhttp.readyState==4) {

			/* 
			 * Palvelimelta on saatu vastaus. Tarkastetaan vastaako vastaus sitä, että sivun lataus on onnistunut (200 OK).
			 * Vastaukseksi voitaisiin saada myös esim. 404 Not Found, jos sivua ei löydy. Nyt tehdään jotain vain, jos sivun lataus onnisui.
			*/
			if (xmlhttp.status==200) {
			
				// Otetaan ylös palvelimelta saatu vastaus, joka on siis ladattu sivu
				var response=xmlhttp.responseText;
				
				// Etsi sarjakuvan URL-osoitetta sivulta
				if (response.search(comicRegExp)!=-1) {
				
					// Lue kuvan osoite
					var imgSrc = RegExp.$1;
						
					if (debug) {
						print (imgSrc);
					}
										
					// Luodaan URL-olio
					urlObj = new URL();
					
					// Määritetään, mihin kuva tallennetaan, kun se on haettu (nopeuttaa huomattavasti)
					urlObj.outputFile = system.widgetDataFolder + "/viivijawagner";
					
					// Asetetaan URL-osoite URL-oliolle
					urlObj.location = imgSrc;
					urlObj.timeout = 20;  // timeoutti 20 sekuntiin
					
					// Haetaan kuva asynkronisesti (ei jää odottamaan kuvan latautumista vaan siirtyy eteenpäin)
					urlObj.fetchAsync(fetchDone);

				} else {
				
					// Jos kuvan osoitetta ei löytynyt, ilmoita virheestä
					lastUpdated.data = widget.getLocalizedString("error2");
					spinner.visible = false;
					fadeIn(comicStrip);
					
					if (debug)
						print ("Img URL error");
				}			

				
			} else {
				// Sivun lataus epäonnistui, kerrotaan käyttäjälle
				lastUpdated.data = widget.getLocalizedString("error");
				spinner.visible = false;
				fadeIn(comicStrip);
			}
		}
	}
      
}


/* Tämä suoritetaan, kun widgetti suljetaan */
function closeAll() {

	// Pysäytä kaikki lataaminen
	urlObj.cancel();
	xmlhttp.abort();

}